@@ -1,22 +1,26 @@
 package ChemOnomatopist::Chain;
 
+# ABSTRACT: Chain of atoms
+# VERSION
+
 use strict;
 use warnings;
 
 use ChemOnomatopist;
+use ChemOnomatopist::Chain::Amide;
+use ChemOnomatopist::Chain::Amine;
 use ChemOnomatopist::Chain::Ether;
 use ChemOnomatopist::Elements qw( %elements );
 use ChemOnomatopist::Group::Carboxyl;
 use ChemOnomatopist::Group::Ether;
+use ChemOnomatopist::Isotope;
+use ChemOnomatopist::Name::Part::AlkaneANSuffix;
 use ChemOnomatopist::Util::SMILES qw( path_SMILES );
 use Graph::Traversal::DFS;
-use List::Util qw( all any sum0 uniq );
+use List::Util qw( all any none sum0 uniq );
 use Scalar::Util qw( blessed );
 use Set::Object qw( set );
 
-# ABSTRACT: Chain of atoms
-# VERSION
-
 sub vertices();
 
 sub new
@@ -29,7 +33,8 @@ sub new
     my $self;
     if( (grep { blessed $_ && $_->isa( ChemOnomatopist::Group::Ether:: ) } @vertices) == 1 ) {
         $self = ChemOnomatopist::Chain::Ether->new( $graph, $parent, @vertices );
-    } elsif( blessed $vertices[0] && $vertices[0]->isa( ChemOnomatopist::Group::Amine:: ) ) {
+    } elsif( blessed $vertices[0] && $vertices[0]->isa( ChemOnomatopist::Group::Amine:: ) &&
+        none { blessed $_ && $_->isa( ChemOnomatopist::Group::Amine:: ) } @vertices[1..$#vertices] ) {
         my $amine = shift @vertices;
         my $chain = bless { vertices => \@vertices, graph => $graph }, $class;
         $self = ChemOnomatopist::Chain::Amine->new( $graph, $chain, $amine );
@@ -170,6 +175,26 @@ sub heteroatom_positions()
 # CHECKME: Can chains have indicated hydrogens?
 sub indicated_hydrogens() { my @hydrogen_positions; return @hydrogen_positions }
 
+sub nonstandard_valence_positions()
+{
+    my( $self ) = @_;
+
+    return @{$self->{nonstandard_valence_positions}} if $self->{nonstandard_valence_positions};
+
+    my @vertices = $self->vertices;
+    my @nonstandard_valence_positions;
+    for (0..$#vertices) {
+        next if blessed $vertices[$_];
+        next if ChemOnomatopist::is_element( $vertices[$_], 'C' );
+        next if ChemOnomatopist::is_element( $vertices[$_], 'N' );
+        next unless exists $vertices[$_]->{valence};
+        push @nonstandard_valence_positions, $_;
+    }
+
+    $self->{nonstandard_valence_positions} = \@nonstandard_valence_positions;
+    return @nonstandard_valence_positions;
+}
+
 sub is_hydrocarbon()
 {
     my( $self ) = @_;
@@ -182,6 +207,12 @@ sub is_saturated()
     return all { $_ eq '-' } $self->bonds;
 }
 
+sub is_substituted()
+{
+    my( $self ) = @_;
+    return $self->number_of_branches > 0;
+}
+
 # Returns maximum number of substitutable locations
 sub max_valence()
 {
@@ -247,6 +278,13 @@ sub multiple_bond_positions()
     return grep { $bonds[$_] =~ /^[=#\$]$/ } 0..$#bonds;
 }
 
+sub double_bond_positions()
+{
+    my( $self ) = @_;
+    my @bonds = $self->bonds;
+    return grep { $bonds[$_] eq '=' } 0..$#bonds;
+}
+
 sub needs_multiple_bond_locants()
 {
     my( $self ) = @_;
@@ -306,6 +344,8 @@ sub needs_suffix_locant()
     # BBv2 P-14.3.4.2 (a): mononuclear parent hydrides do not need locants
     return '' if $self->length == 1;
 
+    return 1 if $self->isotopes;
+
     # BBv2 P-14.3.4.2 (b): monosubstituted homogeneous chains consisting of only two identical atoms do not need locants
     return '' if $self->length == 2 && $self->number_of_branches == 1;
 
@@ -334,6 +374,8 @@ sub needs_substituent_locants()
     return 1;
 }
 
+sub needs_isotope_locants() { &needs_substituent_locants }
+
 sub heteroatoms()
 {
     my( $self ) = @_;
@@ -342,6 +384,42 @@ sub heteroatoms()
                $self->heteroatom_positions;
 }
 
+sub nonstandard_valences()
+{
+    my( $self ) = @_;
+    my @vertices = $self->vertices;
+    return map { $vertices[$_]->{valence} }
+               $self->nonstandard_valence_positions;
+}
+
+sub isotopes()
+{
+    my( $self ) = @_;
+    return @{$self->{isotopes}} if $self->{isotopes};
+
+    my @vertices = $self->vertices;
+    my @isotopes;
+    for my $i (0..$#vertices) {
+        next if blessed $vertices[$i];
+
+        if( exists $vertices[$i]->{isotope} ) {
+            push @isotopes, ChemOnomatopist::Isotope->new( ChemOnomatopist::element( $vertices[$i] ),
+                                                           $vertices[$i]->{isotope},
+                                                           $i,
+                                                           $self->locants( $i ) );
+        }
+        if( exists $vertices[$i]->{h_isotope} ) {
+            for (@{$vertices[$i]->{h_isotope}}) {
+                next unless defined $_;
+                push @isotopes, ChemOnomatopist::Isotope->new( 'H', $_, $i, $self->locants( $i ) );
+            }
+        }
+    }
+
+    $self->{isotopes} = \@isotopes;
+    return @isotopes;
+}
+
 sub length()
 {
     my( $self ) = @_;
@@ -458,12 +536,62 @@ sub number_of_indicated_hydrogens()
     return scalar $self->indicated_hydrogens;
 }
 
+sub number_of_isotopes()
+{
+    my( $self ) = @_;
+    return scalar $self->isotopes;
+}
+
 sub number_of_multiple_bonds()
 {
     my( $self ) = @_;
     return scalar grep { $_ =~ /^[=#\$]$/ } $self->bonds;
 }
 
+sub number_of_nonstandard_valence_positions()
+{
+    my( $self ) = @_;
+    return scalar $self->nonstandard_valence_positions;
+}
+
+sub indicated_hydrogens_part() { ChemOnomatopist::Name->new }
+
+sub isotope_part()
+{
+    my( $self ) = @_;
+
+    my @isotopes = sort { $a->element cmp $b->element ||
+                          $a->atomic_number <=> $b->atomic_number }
+                        $self->isotopes;
+    return '' unless @isotopes;
+
+    my @order;
+    my %freq;
+    for my $isotope (@isotopes) {
+        my $key = $isotope->atomic_number . $isotope->element;
+        if( !$freq{$key} ) {
+            $freq{$key} = [];
+            push @order, $key;
+        }
+        push @{$freq{$key}}, $isotope;
+    }
+
+    my @vertices = $self->vertices;
+    my $isotopes = '';
+    for my $key (@order) {
+        $isotopes .= ',' if $isotopes;
+        $isotopes .= join ',', map { $_->locant } @{$freq{$key}} if $self->needs_isotope_locants;
+        $isotopes .= '-' if $self->needs_isotope_locants;
+        $isotopes .= $key;
+        if( @{$freq{$key}} > 1 ||
+            ( $key =~ /H$/ && $vertices[$freq{$key}->[0]{index}]->{hcount} > 1 ) ) {
+            $isotopes .= scalar @{$freq{$key}};
+        }
+    }
+
+    return "($isotopes)";
+}
+
 sub prefix()
 {
     my( $self ) = @_;
@@ -475,13 +603,69 @@ sub prefix()
         return ChemOnomatopist::Name->new( 'tellan' ) if ChemOnomatopist::is_element( $self->vertices, 'Te' );
     }
 
-    return ChemOnomatopist::unbranched_chain_name( $self ); # FIXME: Add proper suffix
+    return $self->suffix; # FIXME: Add proper suffix
 }
 
 sub suffix()
 {
     my( $self ) = @_;
-    return ChemOnomatopist::unbranched_chain_name( $self );
+
+    my @chain = $self->vertices;
+
+    my $name = ChemOnomatopist::Name->new;
+    if( $self->length == 1 && !blessed $chain[0] && !ChemOnomatopist::is_element( @chain, 'C' ) ) {
+        $name .= 'ne'; # Leaving element prefix appending to the caller
+        return $name;
+    }
+
+    my @bonds = $self->bonds;
+    my @double = grep { $bonds[$_] eq '=' } 0..$#bonds;
+    my @triple = grep { $bonds[$_] eq '#' } 0..$#bonds;
+
+    # BBv2 P-63.2.2.2
+    if( $self->parent && (all { !blessed $_ } @chain) && ChemOnomatopist::is_element( $chain[0], 'O' ) &&
+        !@double && !@triple && all { ChemOnomatopist::is_element( $_, 'C' ) } @chain[1..$#chain] ) {
+        $name->append_stem( ChemOnomatopist::alkane_chain_name( $self->length - 1 ) );
+        $name .= 'oxy';
+        return $name;
+    }
+
+    if( $self->isa( ChemOnomatopist::Chain::Amide:: ) ||
+        $self->isa( ChemOnomatopist::Chain::Amine:: ) ) {
+        $name->append_stem( ChemOnomatopist::alkane_chain_name( scalar grep { !blessed $_ } $self->vertices ) );
+    } elsif( (any { ChemOnomatopist::is_element( $_, 'C' ) } @chain) ||
+        scalar( uniq map { ChemOnomatopist::element( $_ ) } @chain ) > 1 ) {
+        $name->append_stem( ChemOnomatopist::alkane_chain_name( $self->length ) );
+    }
+
+    if( @double ) {
+        $name .= 'a' if @double >= 2; # BBv2 P-16.8.2
+        if( $self->needs_multiple_bond_locants ) {
+            $name->append_locants( $self->bond_locants( @double ) );
+        }
+        if( @double > 1 ) {
+            my $multiplier = ChemOnomatopist::IUPAC_numerical_multiplier( scalar @double );
+            $multiplier .= 'a' unless $multiplier =~ /i$/; # BBv2 P-31.1.1.2
+            $name->append_multiplier( $multiplier );
+        }
+        $name .= 'en';
+    }
+    if( @triple ) {
+        $name .= 'a' if @triple >= 2 && !@double; # BBv2 P-16.8.2
+        if( $self->needs_multiple_bond_locants ) {
+            $name->append_locants( $self->bond_locants( @triple ) );
+        }
+        if( @triple > 1 ) {
+            my $multiplier = ChemOnomatopist::IUPAC_numerical_multiplier( scalar @triple );
+            $multiplier .= 'a' unless $multiplier =~ /i$/; # BBv2 P-31.1.1.2
+            $name->append_multiplier( $multiplier );
+        }
+        $name .= 'yn';
+    }
+
+    $name .= ChemOnomatopist::Name::Part::AlkaneANSuffix->new( 'an' ) unless @double || @triple;
+    $name .= 'e';
+    return $name;
 }
 
 sub vertex_ids
