@@ -1,11 +1,11 @@
 package ChemOnomatopist::Chain::Amine;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Amine chain
 # VERSION
 
+use strict;
+use warnings;
+
 use ChemOnomatopist;
 use ChemOnomatopist::Name;
 use List::Util qw( first );
@@ -23,7 +23,7 @@ sub AUTOLOAD {
 sub new
 {
     my( $class, $graph, $chain, $amine ) = @_;
-    return bless { graph => $graph, chain => $chain, amine => $amine };
+    return bless { graph => $graph, chain => $chain, amine => $amine }, $class;
 }
 
 sub vertices()
@@ -39,17 +39,18 @@ sub locants(@)
     return map { $_ ? $self->{chain}->locants( $_ - 1 ) : 'N' } @_;
 }
 
-sub needs_substituent_locants()
-{
-    my( $self ) = @_;
-    return $self->{chain}->length > 0;
-}
+sub is_hydrocarbon() { '' }
+
+sub needs_substituent_locants() { $_[0]->{chain}->length > 0 }
 
 sub prefix()
 {
     my( $self ) = @_;
     my $prefix = $self->{chain}->prefix;
-    $prefix .= 'amino';
+    pop @$prefix if $prefix->[-1] eq 'e'; # FIXME: Dirty
+    pop @$prefix if $prefix->[-1] eq 'an';
+    $prefix .= 'yl' unless $prefix =~ /[oy]$/;
+    $prefix->append_stem( 'amino' );
     return $prefix;
 }
 
@@ -62,6 +63,7 @@ sub suffix()
 
     my $neighbour = first { $self->graph->has_edge( $self->{amine}, $_ ) }
                           $self->{chain}->vertices;
+    die "cannot perceive connectivity in an amino chain\n" unless defined $neighbour;
 
     return $suffix->append_locants( $self->{chain}->locants( $self->vertex_ids( $neighbour ) ) );
 }
