@@ -1,24 +1,24 @@
 package ChemOnomatopist::Chain::Bicycle;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Fused bicyclic chain
 # VERSION
 
+use strict;
+use warnings;
+
 use ChemOnomatopist;
 use ChemOnomatopist::Chain::Circular;
-use ChemOnomatopist::Elements qw( %elements );
 use ChemOnomatopist::Chain::Bicycle::Purine;
 use ChemOnomatopist::Chain::Monocycle;
 use ChemOnomatopist::Chain::Monocycle::Fused;
+use ChemOnomatopist::Elements qw( %elements );
 use ChemOnomatopist::Name;
 use ChemOnomatopist::Name::Part::Fusion;
 use ChemOnomatopist::Name::Part::Stem;
 use ChemOnomatopist::Util::SMILES qw( cycle_SMILES );
 use Chemistry::OpenSMILES qw( is_double_bond );
 use Graph::Traversal::DFS;
-use List::Util qw( all any min uniq );
+use List::Util qw( all any first min uniq );
 use Scalar::Util qw( blessed );
 use Set::Object qw( set );
 
@@ -54,17 +54,19 @@ our @names = (
 for my $name (qw( 1H-indole indolizine isoindole isoquinoline quinoline quinolizine )) {
     for (grep { $_->[2] eq $name } @names) {
         my @As_parts = @$_;
-        $As_parts[0] =~ s/N/\[As\]/g;
-        $As_parts[1] =~ s/N/\[As\]/g;
+        $As_parts[0] =~ s/n/\[as\]/g;
+        $As_parts[1] =~ s/n/\[as\]/g;
         $As_parts[2] =~ s/^1H-//;
-        $As_parts[2] = 'ars' . $As_parts[2] unless $As_parts[2] =~ s/^iso/isoars/;
+        $As_parts[2] =~ s/^qu//;
+        $As_parts[2] = 'ars' . $As_parts[2] unless $As_parts[2] =~ s/^iso(qu)?/isoars/;
         push @names, \@As_parts;
 
         my @P_parts = @$_;
         $P_parts[0] =~ s/n/p/g;
         $P_parts[1] =~ s/n/p/g;
         $P_parts[2] =~ s/^1H-//;
-        $P_parts[2] = 'phosph' . $P_parts[2] unless $P_parts[2] =~ s/^iso/isophosph/;
+        $P_parts[2] =~ s/^qu//;
+        $P_parts[2] = 'phosph' . $P_parts[2] unless $P_parts[2] =~ s/^iso(qu)?/isophosph/;
         push @names, \@P_parts;
     }
 }
@@ -93,6 +95,10 @@ sub new
     # Each component is represented as an array of vertices in the order of traverse.
     my @components = sort { @$a <=> @$b } $subgraph->connected_components;
     for (0..1) {
+        if( @{$components[$_]} == 1 ) {
+            die "bicycles with three-membered cycles are not supported yet\n"
+        }
+
         my $subgraph = $graph->subgraph( [ @{$components[$_]}, @bridge ] );
         $subgraph->delete_edge( @bridge );
         my @path = Graph::Traversal::DFS->new( $subgraph, start => $bridge[$_] )->dfs;
@@ -157,11 +163,8 @@ sub new
         $self->{cycles} = \@cycles;
         $self->_adjust_vertices_to_cycles;
 
-        if(      join( ',', map { $_->backbone_SMILES } @cycles ) =~ /^c:n:c:n:c:c:,n:c:n:c:c:$/ ||
-                 join( ',', map { $_->backbone_SMILES } @cycles ) =~ /^CNC=Nc:c,n:c:n:c:c:$/ ) {
+        if( $self->is_purine ) {
             return ChemOnomatopist::Chain::Bicycle::Purine->new( $graph, @cycles );
-        } elsif( join( ',', map { $_->backbone_SMILES } @cycles ) =~ /^N=CNCc:c,n:c:n:c:c:$/ ) {
-            return ChemOnomatopist::Chain::Bicycle::Purine->new( $graph, map { $_->flipped } @cycles );
         }
     } elsif( $nbenzene == 1 ) {
         # Numbering has to start from cycle other than benzene
@@ -186,10 +189,11 @@ sub new
 sub candidates()
 {
     my( $self ) = @_;
+    my @chains = ( $self );
 
     if( $self->is_naphthalene ) {
         # Generates all variants
-        my @chains = ( $self, $self->copy, $self->copy, $self->copy );
+        push @chains, $self->copy, $self->copy, $self->copy;
 
         $chains[1]->{cycles} = [ map { $_->flipped } $chains[1]->cycles ];
         $chains[3]->{cycles} = [ map { $_->flipped } $chains[3]->cycles ];
@@ -201,11 +205,9 @@ sub candidates()
             $_->_adjust_vertices_to_cycles;
             $_->{candidate_for} = $self unless $_ == $self;
         }
-
-        return @chains;
     }
 
-    return $self;
+    return @chains;
 }
 
 sub copy()
@@ -279,14 +281,56 @@ sub is_naphthalene()
     return $self->is_hydrocarbon && all { $_->length == 6 } $self->cycles;
 }
 
+sub is_purine()
+{
+    my( $self ) = @_;
+
+    return '' unless $self->number_of_heteroatoms == 4;
+    return '' unless join( ',', uniq $self->heteroatoms ) eq 'N';
+
+    my @cycles = $self->cycles;
+    my $pyrimidine = first { $_->length == 6 } @cycles;
+    my $imidazole  = first { $_->length == 5 } @cycles;
+    return '' unless $pyrimidine && $imidazole;
+
+    return '' unless join( ',', $imidazole->heteroatom_positions ) eq '0,2';
+    return '' unless join( ',', $pyrimidine->heteroatom_positions ) eq '0,2' ||
+                     join( ',', $pyrimidine->heteroatom_positions ) eq '1,3';
+
+    return 1;
+}
+
+sub needs_indicated_hydrogens() { 1 }
+
 sub needs_heteroatom_locants()
 {
     my( $self ) = @_;
     return $self->suffix =~ /^benzo/;
 }
 
-sub needs_heteroatom_names() { return '' } # FIXME: This is not always correct
-sub needs_substituent_locants() { return 1 }
+sub needs_heteroatom_names()
+{
+    my( $self ) = @_;
+    return $self->needs_heteroatom_locants &&
+           all { !$_->is_Hantzsch_Widman } $self->cycles;
+}
+
+sub needs_substituent_locants() { 1 }
+
+sub indicated_hydrogens_part()
+{
+    my( $self ) = @_;
+    return $self->SUPER::indicated_hydrogens_part if $self->is_naphthalene;
+
+    my $part = ChemOnomatopist::Name->new;
+
+    if( $self->number_of_indicated_hydrogens &&
+        $self->number_of_indicated_hydrogens < $self->length ) {
+        $part->append_locants( map { $_ . 'H' } $self->locants( $self->indicated_hydrogens ) );
+    }
+
+    return $part;
+}
 
 sub prefix()
 {
@@ -294,10 +338,10 @@ sub prefix()
 
     my $name = $self->suffix;
     $name = ChemOnomatopist::Name->new( $name ) unless blessed $name;
-    $name->{name}[-1] =~ s/e$//;
+    $name->pop_e;
     if( $self->parent ) { # FIXME: Not stable for naphthalene
         my @vertices = $self->vertices;
-        my( $position ) = grep { $self->graph->has_edge( $self->parent, $vertices[$_] ) } 0..$#vertices;
+        my $position = first { $self->graph->has_edge( $self->parent, $vertices[$_] ) } 0..$#vertices;
         die "unknown locant in multicyclic compound\n" unless defined $position;
         $name->append_substituent_locant( $self->locants( $position ) );
     }
@@ -306,11 +350,9 @@ sub prefix()
     return $name;
 }
 
-# FIXME: This is a bit strange: class and object method with the same name
 sub suffix()
 {
     my( $self ) = @_;
-    return '' unless ref $self;
 
     if( $self->is_hydrocarbon ) {
         # FIXME: Check if aromatic, but with caution, as substitutions will break aromaticity
@@ -327,19 +369,18 @@ sub suffix()
 
     my @SMILES = map { $_->backbone_SMILES } $self->cycles;
     print STDERR "bicycle SMILES: @SMILES\n" if $ChemOnomatopist::DEBUG;
-    my( $retained ) = grep { ($_->[0] eq $SMILES[0] && $_->[1] eq $SMILES[1]) ||
-                             ($_->[0] eq $SMILES[1] && $_->[1] eq $SMILES[0]) } @names;
+    my $retained = first { ($_->[0] eq $SMILES[0] && $_->[1] eq $SMILES[1]) ||
+                           ($_->[0] eq $SMILES[1] && $_->[1] eq $SMILES[0]) } @names;
     return ChemOnomatopist::Name::Part::Stem->new( $retained->[2] )->to_name if $retained;
 
     if( any { $_->is_benzene } $self->cycles ) {
-        my( $other ) = grep { !$_->is_benzene } $self->cycles;
-        $other = ChemOnomatopist::Chain::Monocycle->new( $other->graph, $other->vertices );
+        my $other = first { !$_->is_benzene } $self->cycles;
 
-        my $SMILES = $other->backbone_SMILES;
+        my $SMILES = ChemOnomatopist::Chain::Monocycle->new( $other->graph, $other->vertices )->backbone_SMILES;
         if( $SMILES =~ /^C=C((?<el>O|S|\[Se\]|\[Te\])C|C(?<el>O|S|\[Se\]|\[Te\]))c:c$/ ) {
             # Names according to BBv2 P-25.2.1, Table 2.8, (23) and (24)
             my $element = $+{el};
-            my $name = ($1 =~ /^C/ ? '2H-1-' : '1H-2-') . 'benzo';
+            my $name = 'benzo';
             $element =~ s/[\[\]]//g;
             if( $element ne 'O' ) {
                 $name .= $elements{$element}->{prefix};
@@ -395,8 +436,6 @@ sub suffix()
     my $name = ChemOnomatopist::Name->new;
     my $graph = $self->graph;
 
-    $name->append_locants( map { $_ . 'H' } $self->locants( $self->indicated_hydrogens ) );
-
     my @ideal = map { ChemOnomatopist::Chain::Monocycle->new( $_->graph, $_->vertices ) }
                     $self->cycles;
     my $name_A = $ideal[1]->name;
