@@ -1,11 +1,11 @@
 package ChemOnomatopist::Chain::Carboxamide;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Carboxamide chain
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Chain::;
 
 use List::Util qw( first );
@@ -19,8 +19,8 @@ sub new
                    vertices => [ $amide, $C, $chain->vertices ] }, $class;
 }
 
-sub needs_heteroatom_locants() { return '' }
-sub needs_heteroatom_names() { return '' }
+sub needs_heteroatom_locants() { '' }
+sub needs_heteroatom_names() { '' }
 
 sub locants(@)
 {
@@ -29,7 +29,7 @@ sub locants(@)
 }
 
 # FIXME: This is a source of possible failures
-sub prefix() { return 'benzamido' }
+sub prefix() { 'benzamido' }
 
 sub suffix()
 {
@@ -44,7 +44,10 @@ sub suffix()
                            0..$#vertices;
         $suffix->append_locants( $locant + 1 );
     }
-    $suffix .= 'carbox';
+    $suffix .= 'carbo';
+    if( $self->{vertices}[0]{ketone}->element eq 'O' ) {
+        $suffix .= 'x';
+    }
     return $suffix;
 }
 
