@@ -1,15 +1,16 @@
 package ChemOnomatopist::Chain::Circular;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Chain whose first and last members are connected
 # VERSION
 
+use strict;
+use warnings;
+
 use ChemOnomatopist;
 use ChemOnomatopist::Chain; # FIXME: Not sure why it is needed
-use ChemOnomatopist::Elements qw( %elements );
 use ChemOnomatopist::Chain::Monocycle;
+use ChemOnomatopist::Elements qw( %elements );
+use ChemOnomatopist::Name::Part::NondetachablePrefix;
 use ChemOnomatopist::Util::SMILES qw( cycle_SMILES );
 use Chemistry::OpenSMILES qw( is_single_bond );
 use List::Util qw( all any uniq );
@@ -45,6 +46,14 @@ sub backbone_SMILES()
     return $SMILES;
 }
 
+sub is_Hantzsch_Widman()
+{
+    my( $self ) = @_;
+    return $self->length >= 3 && $self->length <= 10 && # "no more than ten ring members"
+           $self->number_of_heteroatoms &&              # "containing one or more heteroatoms"
+           any { $_->{symbol} =~ /^[cC]$/ } $self->vertices;
+}
+
 # FIXME: What to do with furan and others?
 sub is_aromatic()
 {
@@ -86,6 +95,18 @@ sub is_homogeneous()
     return  1;
 }
 
+sub is_monoreplaced()
+{
+    my( $self ) = @_;
+    return $self->number_of_heteroatoms == 1;
+}
+
+my %five_membered_aromatic_single_heteroatom = (
+    N => '1H-pyrrole', # CHECKME: Do we need to adjust for isomerism?
+    O => 'furan',
+    S => 'thiophene',
+);
+
 sub name()
 {
     my( $self ) = @_;
@@ -95,6 +116,10 @@ sub name()
     my %names = %ChemOnomatopist::Chain::Monocycle::names;
 
     # Check the preserved names
+    if( $self->length == 5 && $self->is_aromatic &&
+        exists $five_membered_aromatic_single_heteroatom{join( '', $self->heteroatoms )} ) {
+        return ChemOnomatopist::Name->new( $five_membered_aromatic_single_heteroatom{join( '', $self->heteroatoms )} );
+    }
     return ChemOnomatopist::Name->new( $names{$SMILES} ) if exists $names{$SMILES};
 
     # Check for aromatic notation
@@ -120,13 +145,12 @@ sub name()
 
     # Check for cycloalkanes
     if( $self->is_hydrocarbon ) {
-        my $name = ChemOnomatopist::Name->new( 'cyclo' );
-        $name .= ChemOnomatopist::unbranched_chain_name( $self );
+        my $name = ChemOnomatopist::Name::Part::NondetachablePrefix->new( 'cyclo' )->to_name;
+        $name .= $self->SUPER::suffix;
         return $name;
     }
 
-    if( $self->length >= 3 && $self->length <= 10 &&
-        any { $_->{symbol} =~ /^[cC]$/ } $self->vertices ) {
+    if( $self->is_Hantzsch_Widman ) {
         # Hantzsch-Widman names (BBv2 P-22.2.2.1)
 
         # Collect the types of heteroatoms and their attachment positions
@@ -159,7 +183,9 @@ sub name()
             if( @{$heteroatoms{$element}} > 1 ) {
                 $name->append_multiplier( ChemOnomatopist::IUPAC_numerical_multiplier( scalar @{$heteroatoms{$element}} ) );
             }
-            $name->append_element( exists $elements{$element}->{HantzschWidman} ? $elements{$element}->{HantzschWidman} : $elements{$element}->{prefix} );
+            $name->append_element( exists $elements{$element}->{HantzschWidman}
+                                        ? $elements{$element}->{HantzschWidman}
+                                        : $elements{$element}->{prefix} );
         }
         $name->[-1] =~ s/a$//;
 
@@ -194,16 +220,18 @@ sub name()
     }
 
     my $name = ChemOnomatopist::Name->new( 'cyclo' );
-    $name .= ChemOnomatopist::unbranched_chain_name( $self );
+    $name .= $self->SUPER::suffix;
     return $name;
 }
 
+sub needs_indicated_hydrogens() { '' }
+
 sub needs_multiple_bond_locants()
 {
     my( $self ) = @_;
-    return 1 if $self->number_of_multiple_bonds > 1;
     return 1 if $self->number_of_branches || $self->parent;
-    return scalar( uniq map { $_->{symbol} } $self->vertices ) > 1;
+    return 1 if scalar( uniq map { $_->{symbol} } $self->vertices ) > 1;
+    return $self->number_of_multiple_bonds > 1 && $self->number_of_multiple_bonds < $self->length;
 }
 
 sub needs_substituent_locants()
@@ -211,18 +239,28 @@ sub needs_substituent_locants()
     my( $self ) = @_;
     # BBv2 P-14.3.4.2 (c): monosubstituted homogeneous cycles do not need locants
     return '' if $self->is_homogeneous && $self->number_of_branches == 1;
-    return '' if $self->is_homogeneous && $self->number_of_branches >= $self->max_valence - 1;
+
+    if( $self->is_homogeneous && $self->number_of_branches >= $self->max_valence - 1 ) {
+        # If there is only one kind of substituents, locants are not needed
+        return '' if uniq( map { "$_" } map { @$_ } $self->locant_names ) == 1;
+    }
     return 1;
 }
 
+sub needs_isotope_locants()
+{
+    my( $self ) = @_;
+    return 1 if $self->number_of_isotopes > 1;
+    return 1 if $self->number_of_branches;
+    return !&is_homogeneous;
+}
+
 sub needs_suffix_locant()
 {
     my( $self ) = @_;
     return $self->needs_substituent_locants;
 }
 
-# sub branch_positions() # TODO: Maybe need to add 1 to all returned positions?
-
 sub bonds()
 {
     my( $self ) = @_;
@@ -248,8 +286,10 @@ sub indicated_hydrogens()
     for my $i (0..$#vertices) {
         # Rough interpretation of BBv2 P-14.7.1 and P-22.2.2.1.4
         next unless $vertices[$i]->{symbol} =~ /^[CN]$/i;
-        next unless $graph->degree( $vertices[$i] ) == 2;
-        next unless all { is_single_bond( $graph, $vertices[$i], $_ ) } @vertices;
+        next unless $graph->degree( $vertices[$i] ) == 2 ||
+                    $graph->degree( $vertices[$i] ) == 3;
+        next unless all { is_single_bond( $graph, $vertices[$i], $_ ) }
+                        $graph->neighbours( $vertices[$i] );
         push @positions, $i;
     }
 
@@ -298,6 +338,25 @@ sub number_of_multiple_bonds()
     return $self->number_of_double_bonds;
 }
 
+sub indicated_hydrogens_part()
+{
+    my( $self ) = @_;
+    my $part = ChemOnomatopist::Name->new;
+    return $part unless $self->number_of_indicated_hydrogens;
+
+    if( $self->needs_indicated_hydrogens ) {
+        my @indicated_hydrogens = $self->indicated_hydrogens;
+        if( $self->number_of_indicated_hydrogens < $self->length ) {
+            $part->append_locants( $self->locants( @indicated_hydrogens ) );
+        }
+        $part .= ChemOnomatopist::IUPAC_numerical_multiplier( scalar @indicated_hydrogens );
+        $part->[-1] .= 'a' unless $part =~ /[ai]$/;
+        $part .= 'hydro';
+    }
+
+    return $part;
+}
+
 sub _cmp_instances
 {
     my( $A, $B ) = @_;
