@@ -1,5 +1,8 @@
 package ChemOnomatopist::Group;
 
+# ABSTRACT: Chemical group
+# VERSION
+
 use strict;
 use warnings;
 
@@ -7,9 +10,13 @@ use ChemOnomatopist::Chain::Circular;
 use ChemOnomatopist::Group::AcylHalide;
 use ChemOnomatopist::Group::Aldehyde;
 use ChemOnomatopist::Group::Amide;
+use ChemOnomatopist::Group::Amidine;
 use ChemOnomatopist::Group::Amine;
+use ChemOnomatopist::Group::Carbaldehyde;
+use ChemOnomatopist::Group::Carbonitrile;
 use ChemOnomatopist::Group::Carboxyl;
 use ChemOnomatopist::Group::Cyanide;
+use ChemOnomatopist::Group::Diazene;
 use ChemOnomatopist::Group::Ester;
 use ChemOnomatopist::Group::Ether;
 use ChemOnomatopist::Group::Guanidine;
@@ -18,16 +25,15 @@ use ChemOnomatopist::Group::Hydrazine;
 use ChemOnomatopist::Group::Hydroperoxide;
 use ChemOnomatopist::Group::Hydroxy;
 use ChemOnomatopist::Group::Imino;
+use ChemOnomatopist::Group::Isocyanate;
+use ChemOnomatopist::Group::Isocyanide;
 use ChemOnomatopist::Group::Ketone;
 use ChemOnomatopist::Group::SulfinicAcid;
 use ChemOnomatopist::Group::SulfonicAcid;
-
+use ChemOnomatopist::Group::Urea;
 use List::Util qw( any );
 use Scalar::Util qw( blessed );
 
-# ABSTRACT: Chemical group
-# VERSION
-
 # Order from BBv2 P-41
 our @order = (
     # Radicals
@@ -45,17 +51,23 @@ our @order = (
     ChemOnomatopist::Group::Ester::,
     # Acid halides and pseudohalides
     # Amides
+    ChemOnomatopist::Group::Urea::,
     ChemOnomatopist::Group::Amide::,
     ChemOnomatopist::Group::Guanidine::,
     # Hydrazides
     ChemOnomatopist::Group::Hydrazide::,
     # Imides
     # 14. Nitriles
+    ChemOnomatopist::Group::Carbonitrile::,
     ChemOnomatopist::Group::Cyanide::,
+    ChemOnomatopist::Group::Isocyanate::, # CHECKME: Is this correct?
+    ChemOnomatopist::Group::Isocyanide::, # CHECKME: Is this correct?
+    ChemOnomatopist::Group::Carbaldehyde::, # CHECKME: How is this related to ChemOnomatopist::Group::Aldehyde?
     ChemOnomatopist::Group::Aldehyde::,
     ChemOnomatopist::Group::Ketone::,
     ChemOnomatopist::Group::Hydroxy::,
     ChemOnomatopist::Group::Hydroperoxide::,
+    ChemOnomatopist::Group::Amidine::, # CHECKME: Is this correct?
     ChemOnomatopist::Group::Amine::,
     ChemOnomatopist::Group::Imino::,
 
@@ -64,6 +76,7 @@ our @order = (
     # TODO: Classes denoted by the senior atom in heterane nomenclature should go here
     # 21. Nitrogen compounds
     ChemOnomatopist::Group::Hydrazine::,
+    ChemOnomatopist::Group::Diazene::,
     # 41. Ethers, then sulfides, sulfoxides, sulfones; then selenides, selenoxides, etc.
     ChemOnomatopist::Group::Ether::,
 );
@@ -74,7 +87,7 @@ sub new
     return bless { element => $element }, $class;
 }
 
-sub element() { return $_[0]->{element} }
+sub element() { $_[0]->{element} }
 
 sub is_carbon()
 {
@@ -94,22 +107,22 @@ sub is_oxygen()
     return $self->element && $self->element eq 'O';
 }
 
-sub is_part_of_chain() { return '' }
+sub is_part_of_chain() { '' }
 
 # Certain groups can only be expressed as prefixes
-sub is_prefix_only() { return '' }
+sub is_prefix_only() { '' }
 
 # Certain groups can only be terminal in chains
-sub is_terminal() { return '' }
+sub is_terminal() { '' }
 
-sub needs_heteroatom_locants { return 1 }
-sub needs_heteroatom_names { return 1 }
-sub needs_multiple_bond_suffix { return 1 }
+sub needs_heteroatom_locants   { 1 }
+sub needs_heteroatom_names     { 1 }
+sub needs_multiple_bond_suffix { 1 }
 
-sub prefix() { return '' }
-sub suffix() { return $_[0]->is_prefix_only ? undef : '' }
-sub multisuffix() { return $_[0]->suffix }
-sub suffix_if_cycle_substituent() { return $_[0]->suffix }
+sub prefix() { '' }
+sub suffix() { $_[0]->is_prefix_only ? undef : '' }
+sub multisuffix() { $_[0]->suffix }
+sub suffix_if_cycle_substituent() { $_[0]->suffix }
 
 sub candidate_for()
 {
@@ -179,6 +192,6 @@ sub cmp
 }
 
 # Two instances of the same group are thought to be of the same seniority
-sub _cmp_instances { return 0 }
+sub _cmp_instances { 0 }
 
 1;
