@@ -1,11 +1,11 @@
 package ChemOnomatopist::Group::Aldehyde;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Aldehyde group
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Group::;
 
 sub new
@@ -14,21 +14,22 @@ sub new
     return bless { ketone => $ketone }, $class;
 }
 
-sub element { return 'C' }
+sub element() { 'C' }
 
-sub is_part_of_chain() { return 1 }
+sub is_part_of_chain() { 1 }
 
-sub prefix { return 'formyl' }
+sub prefix { 'formyl' }
 
 sub suffix()
 {
     my( $self ) = @_;
     my $name = $self->{ketone}->suffix;
     $name =~ s/one$/al/;
+    $name = 'selenal' if $name eq 'selal';
     return $name;
 }
 
-sub multisuffix { return 'carbaldehyde' }
-sub suffix_if_cycle_substituent { return 'carbaldehyde' }
+sub multisuffix { 'carbaldehyde' }
+sub suffix_if_cycle_substituent { 'carbaldehyde' }
 
 1;
