@@ -1,16 +1,15 @@
 package ChemOnomatopist::Group::Cyanide;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Cyanide group
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Group::;
 
-# FIXME: When part of chain, suffix is 'nitrile'; when an attachment - 'carbonitrile'.
-
-sub prefix { return 'cyano' }
-sub suffix { return 'carbonitrile' }
+sub prefix() { 'cyano' }
+sub suffix() { 'nitrile' }
+sub multisuffix() { 'carbonitrile' }
 
 1;
