@@ -1,17 +1,17 @@
 package ChemOnomatopist::Group::Hydrazide;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Hydrazide group
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Group::, ChemOnomatopist::Chain::;
 
 sub new
 {
-    my( $class, $graph, @vertices ) = @_;
-    return bless { graph => $graph, vertices => \@vertices }, $class;
+    my( $class, $graph, $ketone, @vertices ) = @_;
+    return bless { graph => $graph, ketone => $ketone, vertices => \@vertices }, $class;
 }
 
 sub needs_heteroatom_locants() { return '' }
@@ -28,16 +28,9 @@ sub locants(@)
     return map { $_ == 0 ? "N'" : $_ == 1 ? 'N' : $_ - 1 } @_;
 }
 
-sub prefix()
-{
-    my( $self ) = @_;
-    return 'hydrazidyl';
-}
+my %suffixes = ( O => '', S => 'thio', Se => 'seleno', Te => 'telluro' );
 
-sub suffix()
-{
-    my( $self ) = @_;
-    return 'hydrazide';
-}
+sub prefix() { $suffixes{$_[0]->{ketone}->element} . 'hydrazidyl' }
+sub suffix() { $suffixes{$_[0]->{ketone}->element} . 'hydrazide' }
 
 1;
