@@ -1,13 +1,16 @@
 package ChemOnomatopist::Group::Hydrazine;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Hydrazine group
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Group::, ChemOnomatopist::Chain::;
 
+use Chemistry::OpenSMILES qw( is_double_bond );
+use List::Util qw( any );
+
 sub new
 {
     my( $class, $graph, @vertices ) = @_;
@@ -24,24 +27,21 @@ sub candidates()
     return @chains;
 }
 
-sub needs_heteroatom_locants() { return '' }
-sub needs_heteroatom_names() { return '' }
+sub needs_heteroatom_locants() { '' }
+sub needs_heteroatom_names() { '' }
 sub needs_substituent_locants()
 {
     my( $self ) = @_;
+
+    for my $vertex ($self->vertices) {
+        return '' if any { is_double_bond( $self->graph, $vertex, $_ ) }
+                         $self->graph->neighbours( $vertex );
+    }
+
     return $self->number_of_branches > 1 && $self->number_of_branches < $self->max_valence;
 }
 
-sub prefix()
-{
-    my( $self ) = @_;
-    return 'hydrazinyl';
-}
-
-sub suffix()
-{
-    my( $self ) = @_;
-    return 'hydrazine';
-}
+sub prefix() { 'hydrazinyl' }
+sub suffix() { 'hydrazine' }
 
 1;
