@@ -1,16 +1,17 @@
 package ChemOnomatopist::Name;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Chemical name
 # VERSION
 
+use strict;
+use warnings;
+
+use overload '.'   => \&concatenate;
 use overload '.='  => \&append;
-use overload '""'  => sub { return join '', @{$_[0]->{name}} };
-use overload 'eq'  => sub { return  "$_[0]" eq  "$_[1]" };
-use overload 'cmp' => sub { return ("$_[0]" cmp "$_[1]") * ($_[2] ? -1 : 1) };
-use overload '@{}' => sub { return $_[0]->{name} };
+use overload '""'  => sub { join '', @{$_[0]->{name}} };
+use overload 'eq'  => sub {  "$_[0]" eq  "$_[1]" };
+use overload 'cmp' => sub { ("$_[0]" cmp "$_[1]") * ($_[2] ? -1 : 1) };
+use overload '@{}' => sub { $_[0]->{name} };
 
 use ChemOnomatopist::Name::Part::AlkaneANSuffix;
 use ChemOnomatopist::Name::Part::Element;
@@ -30,7 +31,7 @@ sub new
     if( defined $name && blessed $name ) {
         push @name_parts, $name;
     } elsif( defined $name && $name ne '' ) {
-        push @name_parts, $name;
+        push @name_parts, ChemOnomatopist::Name::Part::Stem->new( $name );
     }
     return bless { name => \@name_parts }, $class;
 }
@@ -39,20 +40,14 @@ sub new
 sub append($)
 {
     my( $self, $name ) = @_;
+    return $self if $name eq ''; # Do not do anything if name is empty
 
     $self->[-1] =~ s/a$// if $name =~ /^a/ && @$self;
     $self->[-1] =~ s/o$// if $name =~ /^o/ && @$self && $self->[-1] ne 'cyclo';
 
-    # BBv2 P-16.7.1 (d)
-    my $terminal_element;
-    if( @$self && blessed $self->[-1] && $self->[-1]->isa( ChemOnomatopist::Name::Part::Element:: ) &&
-        "$name" =~ /^[aeiouy]/ ) {
-        $self->[-1] =~ s/a$//;
-    }
-
     # If names are combined and the second one starts with a number, a separator is added.
-    if( @$self && blessed $name && $name->isa( ChemOnomatopist::Name:: ) && $name =~ /^\d/ ) {
-        push @$self, '-';
+    if( @$self && !$self->ends_with_locant && blessed $name && $name->isa( ChemOnomatopist::Name:: ) && $name =~ /^\d/ ) {
+        $name->[0]->{value} = '-' . $name->[0]->{value};
     }
     # FIXME: The following needlessly converts $name into string
     $name =~ s/^-// if @$self && $self->[-1] =~ /-$/ && $name =~ /^-/;
@@ -73,7 +68,7 @@ sub append_locants
     my( $self, @locants ) = @_;
     return $self unless @locants;
 
-    if( @$self ) {
+    if( @$self && !$self->ends_with_locant ) {
         $self->append( ChemOnomatopist::Name::Part::Locants->new( '-' . join( ',', @locants ) . '-' ) );
     } else {
         $self->append( ChemOnomatopist::Name::Part::Locants->new( join( ',', @locants ) . '-' ) );
@@ -155,11 +150,35 @@ sub bracket_numeric_locants()
     for my $i (1..$#$self) {
         next unless blessed $self->[$i];
         next unless $self->[$i]->isa( ChemOnomatopist::Name::Part::Locants:: );
-        $self->[$i-1] = '[';
-        $self->[$i]->{value} =~ s/-$/]/;
+        $self->[$i]->{value} =~ s/^-//;
+        $self->[$i]->{value} =~ s/-$//;
+        $self->[$i]->{value} = '[' . $self->[$i]->{value} . ']';
     }
 }
 
+sub concatenate($$@)
+{
+    my( $A, $B, $reverse ) = @_;
+    ( $A, $B ) = ( $B, $A ) if $reverse;
+    $A = blessed $A ? clone( $A ) : ChemOnomatopist::Name->new( $A );
+    return $A->append( $B );
+}
+
+sub pop_e()
+{
+    my( $self ) = @_;
+    return $self unless @$self;
+
+    if( blessed $self->[-1] ) {
+        $self->[-1]{value} =~ s/e$//;
+        pop @$self if $self->[-1]{value} eq '';
+    } else {
+        $self->[-1] =~ s/e$//;
+        pop @$self if $self->[-1] eq '';
+    }
+    return $self;
+}
+
 sub has_locant()
 {
     my( $self ) = @_;
@@ -225,6 +244,14 @@ sub starts_with_multiplier()
            $self->[0]->isa( ChemOnomatopist::Name::Part::Multiplier:: );
 }
 
+sub ends_with_locant()
+{
+    my( $self ) = @_;
+    return @$self &&
+           blessed $self->[-1] &&
+           $self->[-1]->isa( ChemOnomatopist::Name::Part::Locants:: );
+}
+
 sub ends_with_multiplier()
 {
     my( $self ) = @_;
