@@ -1,11 +1,11 @@
 package ChemOnomatopist::Util::Graph;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Generic graph utilities
 # VERSION
 
+use strict;
+use warnings;
+
 use ChemOnomatopist::Util qw( copy );
 use Exporter;
 use Graph::Traversal::BFS;
@@ -25,7 +25,6 @@ our @EXPORT_OK = qw(
     graph_longest_paths
     graph_longest_paths_from_vertex
     graph_path_between_vertices
-    graph_replace
     graph_without_edge_attributes
     merge_graphs
     subgraph
@@ -162,6 +161,7 @@ sub graph_cycles
 sub graph_has_cycle
 {
     my( $graph ) = @_;
+    return '' unless $graph->vertices;
     return $graph->edges > $graph->vertices - 1;
 }
 
@@ -255,28 +255,6 @@ sub graph_longest_paths_from_vertex
     return @longest_paths;
 }
 
-# Replace one or more old vertices with a single new one
-sub graph_replace
-{
-    my( $graph, $new, @old ) = @_;
-
-    $graph->add_vertex( $new );
-
-    my $old = set( @old );
-    for my $edge (grep { ($old->has( $_->[0] ) && !$old->has( $_->[1] )) ||
-                         ($old->has( $_->[1] ) && !$old->has( $_->[0] )) }
-                       $graph->edges) {
-        my( $vertex, $neighbour ) = $old->has( $edge->[0] ) ? @$edge : reverse @$edge;
-        next if $graph->has_edge( $new, $neighbour );
-        $graph->add_edge( $new, $neighbour );
-        next unless $graph->has_edge_attributes( @$edge );
-        $graph->set_edge_attributes( $new, $neighbour, $graph->get_edge_attributes( @$edge ) );
-    }
-    $graph->delete_vertices( @old );
-
-    return $graph;
-}
-
 sub graph_without_edge_attributes($)
 {
     my( $graph ) = @_;
@@ -308,6 +286,10 @@ sub subgraph
     my $vertices = set( @vertices );
     $graph = copy $graph;
     $graph->delete_vertices( grep { !$vertices->has( $_ ) } $graph->vertices );
+    # For some reason bridges() have to be called to update the cache.
+    # Otherwise is_edge_connected() may return an incorrect result.
+    # This should be reported upstream, but MWE is difficult to construct.
+    $graph->bridges;
     return $graph;
 }
 
